using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;

namespace CorsServer.WebApi31
{
    public class Startup
    {
        public Startup(IConfiguration configuration, IHostEnvironment hostingEnvironment, IWebHostEnvironment webHostEnvironment)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        public void ConfigureServices(IServiceCollection services)
        {
            //Config
            services.Configure<CorsOption>(Configuration.GetSection(ApplicationConst.CorsConfigOptionName));

            #region  CORS
            AddCors_Config(services);
            //AddCors_Test(services);
            //AddCors_2(services);
            //AddCors_3(services);
            //AddCors_4(services);
            //AddCors_5(services);
            #endregion
            services.AddControllers();
        }

        public void Configure(IApplicationBuilder app, IWebHostEnvironment env, IOptionsSnapshot<CorsOption> corsOtionsSnapshot)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
                //app.UseDatabaseErrorPage();
            }
            else
            {
                //app.UseExceptionHandler("/Error");
                //app.UseHsts();
            }

            // app.UseHttpsRedirection();

            #region ѹͰھ̬ļ
            // app.UseResponseCompression();
            // app.UseResponseCaching();
            #endregion

            // app.UseStaticFiles();

            // app.UseCookiePolicy();

            // ·ȫַǰ http://www.custom.com/PathBase/
            // app.UsePathBase("/api/");

            app.UseRouting();

            // app.UseRequestLocalization();

            // UseRouting()  UseAuthentication()֮
            // м˳ https://docs.microsoft.com/zh-cn/aspnet/core/fundamentals/middleware/?view=aspnetcore-6.0#middleware-order
            app.UseCors(ApplicationConst.DefaultPolicyName);

            //app.UseAuthentication();

            app.UseAuthorization();

            // Cookieм֮ MVCм֮ǰûỰм
            // app.UseSession();

            // app.UseResponseCompression();

            // app.UseResponseCaching();

            //Զм

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });
        }

        /// <summary>
        /// CORS ģ
        /// </summary>
        private IServiceCollection AddCors_Template(IServiceCollection services)
        {
            services.AddCors(setup =>
            {
                var corsOption = services.BuildServiceProvider().GetRequiredService<IOptionsSnapshot<CorsOption>>().Value;
                setup.AddPolicy(ApplicationConst.DefaultPolicyName, build =>
                {
                    build

                    //Դ
                    //1Դ
                    .AllowAnyOrigin()

                    //2lamdaԶ
                    //.SetIsOriginAllowed(requestOrigin => 
                    //{
                    //    //Դ(Э++˿ںţ http://wwwww.xxxx.com:80)
                    //    var address = requestOrigin;

                    //    //Դй

                    //    return true;
                    //})

                    //3WithOriginsԶ
                    //.WithOrigins(corsOption.Origins.ToArray())
                    //WithOriginsʹͨ(*)
                    //.SetIsOriginAllowedToAllowWildcardSubdomains()

                    //󷽷(POST GET PUT DELETE OPTIONS)
                    .AllowAnyMethod()
                    //.WithMethods(corsOption.Methods.ToArray())

                    //ͷ
                    .AllowAnyHeader()
                    //.WithHeaders(corsOption.Headers.ToArray())

                    //ƾ
                    //.AllowCredentials()
                    //.DisallowCredentials()

                    //.WithExposedHeaders()
                    //.SetPreflightMaxAge(TimeSpan.FromMinutes(10))
                   ;
                });
            });

            return services;
        }

        /// <summary>
        /// ȫ˵
        /// </summary>
        private IServiceCollection AddCors_Info(IServiceCollection services)
        {
            services.AddCors(setup =>
            {
                setup.AddPolicy(ApplicationConst.DefaultPolicyName, build =>
                {
                    build

                    //Դ
                    .AllowAnyOrigin()                                   //κԴ
                    //.SetIsOriginAllowed(_=> true)                    //ʹFunc<string bool> ίзԶǷԴ
                    //.WithOrigins()                                    //ָԴ
                    //.SetIsOriginAllowedToAllowWildcardSubdomains()    //WithOrigins()Դʹͨ(*) 

                    //HTTP(POST GET PUT DELETE OPTIONS)
                    .AllowAnyMethod()                                   //з
                    //.WithMethods()                                    //ָ

                    //ͷ
                    .AllowAnyHeader()                                   //ͷ
                                                                        //.WithHeaders()                                    //ָͷ

                    //еƾ
                    .AllowCredentials()                                 //ƾݣ֤а(cookies)HTTP֤Э(HTTP authentication schemes)
                    //.DisallowCredentials()                            //ܾƾ

                    //ùķǼӦͷ                          /ñ¶ԶӦͷĬ£ֻᱩ¶ĬϵӦͷӦãԶӰͷᱩ¶Ӧó
                    .WithExposedHeaders("x-custom-a", "x-custom-b")

                    //Ԥʱ
                    .SetPreflightMaxAge(TimeSpan.FromMinutes(10))  //˱ͷָɻԤӦʱ䳤
                    ;

                    /*ر˵
                      ڰȫǣ.net core 2.1ʼ, AllowAnyOrigin()  AllowCredentials() ͬʱʹ
                      
                                1ʹAllowCredentials()ʱ.SetIsOriginAllowed(_ => true)  AllowAnyOrigin()
                                2ʹAllowCredentials()ʱ WithOrigins()ָԴ(ʹSetIsOriginAllowedToAllowWildcardSubdomains()ͨ)  AllowAnyOrigin()
                                3Զм
                     */
                });
            });

            return services;
        }

        #region ע᲻ͬCors

        /// <summary>
        /// 
        /// </summary>
        private IServiceCollection AddCors_Test(IServiceCollection services)
        {
            services.AddCors(setup =>
            {
                setup.AddPolicy(ApplicationConst.DefaultPolicyName, build =>
                {
                    build

                    //Դ
                    //.AllowAnyOrigin()
                    //.WithOrigins()
                    .SetIsOriginAllowed(requestOrigin =>
                    {
                        var cc = requestOrigin;
                        return true;
                    })
                    //.SetIsOriginAllowedToAllowWildcardSubdomains()

                    //󷽷(POST GET PUT DELETE OPTIONS)
                    .AllowAnyMethod()
                    //.WithMethods()

                    //ͷ
                    .AllowAnyHeader()
                    //.WithHeaders()

                    //ƾ
                    //.AllowCredentials()
                    //.DisallowCredentials()

                    //.WithExposedHeaders()
                   ;
                });
            });

            return services;
        }

        private IServiceCollection AddCors_Single(IServiceCollection services)
        {
            services.AddCors(setup =>
             {
                 setup.AddPolicy(ApplicationConst.DefaultPolicyName, build =>
                 {
                     build

                     //Դ
                     .AllowAnyOrigin()
                    //.WithOrigins()
                    //.SetIsOriginAllowed(_ => true)
                    //.SetIsOriginAllowedToAllowWildcardSubdomains()

                    //󷽷(POST GET PUT DELETE OPTIONS)
                    //.AllowAnyMethod()
                    //.WithMethods()

                    //ͷ
                    //.AllowAnyHeader()
                    //.WithHeaders()

                    //ƾ
                    //.AllowCredentials()
                    //.DisallowCredentials()

                    //.WithExposedHeaders()
                    ;
                 });
             });

            return services;
        }

        private IServiceCollection AddCors_All(IServiceCollection services)
        {
            services.AddCors(setup =>
           {
               setup.AddPolicy(ApplicationConst.DefaultPolicyName, build =>
               {
                   build
                   .AllowAnyOrigin()
                   .AllowAnyMethod()
                   .AllowAnyHeader()
                   ;
               });
           });

            return services;
        }

        private IServiceCollection AddCors_3(IServiceCollection services)
        {
            services.AddCors(setup =>
             {
                 var corsOption = services.BuildServiceProvider().GetRequiredService<IOptionsSnapshot<CorsOption>>().Value;
                 setup.AddPolicy(ApplicationConst.DefaultPolicyName, build =>
                 {
                     build
                     .WithOrigins(corsOption.Origins.ToArray())
                     .WithMethods(corsOption.Methods.ToArray())
                     .WithHeaders(corsOption.Headers.ToArray())
                     .WithExposedHeaders(corsOption.ExposedHeaders.ToArray());
                 });
             });

            return services;
        }

        private IServiceCollection AddCors_4(IServiceCollection services)
        {
            services.AddCors(setup =>
           {
               setup.AddPolicy(ApplicationConst.DefaultPolicyName, build =>
               {
                   build.AllowAnyOrigin().AllowAnyMethod().AllowAnyHeader().WithExposedHeaders("x-custom-error");
               });
           });
            return services;
        }

        private IServiceCollection AddCors_5(IServiceCollection services)
        {
            services.AddCors(setup =>
              {
                  var corsOption = services.BuildServiceProvider().GetRequiredService<IOptionsSnapshot<CorsOption>>().Value;
                  setup.AddPolicy(ApplicationConst.DefaultPolicyName, builder =>
                  {
                      builder
                      ////.SetIsOriginAllowedToAllowWildcardSubdomains()
                      .WithOrigins("http://localhost:5002")
                      //.AllowAnyMethod()
                      ////.WithMethods(corsOption.Methods.ToArray())
                      //.AllowAnyHeader()
                      ////.WithHeaders(corsOption.Headers.ToArray())
                      ////.WithExposedHeaders(corsOption.ExposedHeaders.ToArray())
                      ;
                  });
              });

            return services;
        }

        private IServiceCollection AddCors_Config(IServiceCollection services)
        {
            services.AddCors(setup =>
              {
                  var corsOption = services.BuildServiceProvider().GetRequiredService<IOptionsSnapshot<CorsOption>>().Value;
                  setup.AddPolicy(ApplicationConst.DefaultPolicyName, builder =>
                  {
                      if (corsOption.Origins == null)
                      {
                          builder.SetIsOriginAllowed(_ => true);
                      }
                      else if (corsOption.Origins.Count == 0)
                      {
                          builder.SetIsOriginAllowed(_ => true);
                      }
                      else if (corsOption.Origins.Contains("*"))
                      {
                          builder.SetIsOriginAllowed(_ => true);
                      }
                      else
                      {
                          builder.WithOrigins(corsOption.Origins.ToArray());
                          builder.SetIsOriginAllowedToAllowWildcardSubdomains();
                      }

                      if (corsOption.Methods == null || corsOption.Methods.Count == 0)
                      {
                          builder.AllowAnyMethod();
                      }
                      else if (corsOption.Methods.Contains("*"))
                      { 
                          builder.AllowAnyMethod();
                      }
                      else
                      {
                          builder.WithMethods(corsOption.Methods.ToArray());
                      }

                      if (corsOption.Headers == null || corsOption.Headers.Count == 0)
                      {
                          builder.AllowAnyHeader();
                      }

                      else if (corsOption.Headers.Contains("*"))
                      {
                          builder.AllowAnyHeader();
                      }
                      else
                      {
                          builder.WithMethods(corsOption.Headers.ToArray());
                      }

                      if (corsOption.ExposedHeaders != null && corsOption.ExposedHeaders.Count > 0)
                      {
                          builder.WithExposedHeaders(corsOption.ExposedHeaders.ToArray());
                      }

                      if (corsOption.AllowCredentials)
                      {
                          builder.AllowCredentials();
                      }
                      else
                      {
                          builder.DisallowCredentials();
                      }

                      if (corsOption.PreflightMaxAge.TotalSeconds > 0)
                      {
                          builder.SetPreflightMaxAge(corsOption.PreflightMaxAge);
                      }
                  });
              });

            return services;
        }

        #endregion
    }
}
